@@ -130,9 +130,8 @@ def isStudy(dirpath):
     """Try to determine if dirpath is a Code_Saturne study directory.
     """
 
-    postd = os.path.join(dirpath, 'POST')
     meshd = os.path.join(dirpath, 'MESH')
-    is_study = os.path.isdir(postd) and os.path.isdir(meshd)
+    is_study = os.path.isdir(meshd)
 
     return is_study
 
@@ -266,14 +265,14 @@ class Case(object):
                     case.xmlSaveDocument()
 
 
-        # 2) Create RESU and SRC directory if needed
+        # 2) Recreate missing directories which are compulsory
+        # git removes these usually empty directories
         if not xmlonly:
-            r = os.path.join(self.__repo, subdir, "RESU")
-            if not os.path.isdir(r):
-                os.makedirs(r)
-            r = os.path.join(self.__repo, subdir, "SRC")
-            if not os.path.isdir(r):
-                os.makedirs(r)
+            git_rm_dirs = ["RESU", "SRC"]
+            for gd in git_rm_dirs:
+                r = os.path.join(self.__repo, subdir, gd)
+                if not os.path.isdir(r):
+                    os.makedirs(r)
 
         # 3) Update the GUI script from the Repository
         data_subdir = os.path.join(self.__repo, subdir, "DATA")
@@ -373,11 +372,16 @@ class Case(object):
         for d in cdirs:
             self.__update_domain(d, xmlonly)
 
-        # Update the runcase script from the Repository in case of coupling
+        # Update the runcase script from the repository in case of coupling
 
         if self.subdomains:
             case_dir = os.path.join(self.__repo, self.label)
             self.update_runcase_path(case_dir, None, xmlonly)
+            # recreate possibly missing but compulsory directory
+            r = os.path.join(case_dir, "RESU_COUPLING")
+            if not os.path.isdir(r):
+                os.makedirs(r)
+
 
     #---------------------------------------------------------------------------
 
@@ -1113,15 +1117,15 @@ class Studies(object):
             init_xml_file_with_study(smgr, studyp)
 
         elif options.create_xml and not is_study:
-            print("Can not create XML file of parameter:\n" \
-                  + "current directory is not a study.")
-            sys.exit(1)
+            msg =   "Can not create XML file of parameter:\n" \
+                  + "current directory is apparently not a study (no MESH directory).\n"
+            sys.exit(msg)
 
         if filename == None:
-            print("A file of parameters must be specified or created " \
-                  "for studymanager to run.\n"
-                  "See help message and use '--file' or '--create-xml' option.")
-            sys.exit(1)
+            msg =    "A file of parameters must be specified or created " \
+                   + "for studymanager to run.\n" \
+                   + "See help message and use '--file' or '--create-xml' option.\n"
+            sys.exit(msg)
 
         # create a first xml parser only for
         #   the repository verification and
@@ -1130,8 +1134,8 @@ class Studies(object):
         if os.path.isfile(filename):
             self.__parser = Parser(filename)
         else:
-            print("Specified XML parameter file for studymanager does not exist.")
-            sys.exit(1)
+            msg = "Specified XML parameter file for studymanager does not exist.\n"
+            sys.exit(msg)
 
         self.__xmlupdate = options.update_xml
 
@@ -1151,9 +1155,10 @@ class Studies(object):
                 self.__parser.setRepository(os.path.join(studyp,".."))
                 self.__repo = self.__parser.getRepository()
             else:
-                msg="Parser.getRepository() >> No repository set.\n"
-                msg+="Add a path to the *.xml file or use the command "
-                msg+="line argument.\n"
+                msg =   "Can not set a default repository directory:\n" \
+                      + "current directory is apparently not a study (no MESH directory).\n" \
+                      + "Add a repository path to the parameter file or use the command " \
+                      + "line option (--repo=..).\n"
                 sys.exit(msg)
 
         # set destination
@@ -1171,10 +1176,11 @@ class Studies(object):
                                                               "../RUN_"+studyd))
                     self.__dest = self.__parser.getDestination()
                 else:
-                    msg="Parser.getDestination() >> No destination set.\n"
-                    msg+="Add a path to the *.xml file or use the command "
-                    msg+="line argument.\n"
-                    sys.exit(msg)
+                  msg =   "Can not set a default destination directory:\n" \
+                        + "current directory is apparently not a study (no MESH directory).\n" \
+                        + "Add a destination path to the parameter file or use the command " \
+                        + "line option (--dest=..).\n"
+                  sys.exit(msg)
 
         # create if necessary the destination directory
 
@@ -1473,7 +1479,7 @@ class Studies(object):
                 if self.__running:
                     if case.compute == 'on' and case.is_compiled != "KO":
 
-                        if self.__n_iter:
+                        if self.__n_iter is not None:
                             if case.subdomains:
                                 case_dir = os.path.join(self.__dest, s.label, case.label,
                                                         case.subdomains[0], "DATA")
