@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -58,7 +58,7 @@ typedef struct _fvm_to_ensight_case_t fvm_to_ensight_case_t;
 typedef struct {
 
   const char  * name;    /* Pointer to file name */
-  _Bool         queried; /* Indicates if this file name has already been
+  bool          queried; /* Indicates if this file name has already been
                             returned by "fvm_to_ensight_case_get_..._file()"
                             (so we can decide to create or append to it) */
 
