@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -25,11 +25,9 @@
 !------------------------------------------------------------------------------
 !   mode          name          role
 !------------------------------------------------------------------------------
-!> \param[out]    nmodpp        number of activated paricle physic models
 !______________________________________________________________________________
 
-subroutine ppvarp ( nmodpp )
-!================
+subroutine ppvarp
 
 !===============================================================================
 !  FONCTION  :
@@ -39,16 +37,6 @@ subroutine ppvarp ( nmodpp )
 !   LE TYPE DE PHYSIQUE PARTICULIERE
 ! REMPLISSAGE DES PARAMETRES (DEJA DEFINIS) POUR LES SCALAIRES PP
 
-!-------------------------------------------------------------------------------
-! Arguments
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-!__________________!____!_____!________________________________________________!
-
-!     Type: i (integer), r (real), s (string), a (array), l (logical),
-!           and composite types (ex: ra real array)
-!     mode: <-- input, --> output, <-> modifies data, --- work array
 !===============================================================================
 
 !===============================================================================
@@ -67,12 +55,13 @@ use ppthch
 use coincl
 use cpincl
 use ppincl
-use elincl
 use cs_coal_incl
 use cs_fuel_incl
 use ppcpfu
 use atincl
 use ihmpre
+use field
+use cs_c_bindings
 
 !===============================================================================
 
@@ -80,9 +69,52 @@ implicit none
 
 ! Arguments
 
-integer       nmodpp
+! Local variables
+
+integer          f_id
+integer          kscmin, kscmax
+
+double precision scmaxp, scminp
 
 !===============================================================================
+! Interfaces
+!===============================================================================
+
+interface
+
+  subroutine cs_elec_add_variable_fields()  &
+    bind(C, name='cs_elec_add_variable_fields')
+    use, intrinsic :: iso_c_binding
+    implicit none
+  end subroutine cs_elec_add_variable_fields
+
+end interface
+
+interface
+
+  subroutine cs_field_pointer_map_gas_mix()  &
+    bind(C, name='cs_field_pointer_map_gas_mix')
+    use, intrinsic :: iso_c_binding
+    implicit none
+  end subroutine cs_field_pointer_map_gas_mix
+
+end interface
+
+interface
+
+  subroutine cs_field_pointer_map_groundwater()  &
+    bind(C, name='cs_field_pointer_map_groundwater')
+    use, intrinsic :: iso_c_binding
+    implicit none
+  end subroutine cs_field_pointer_map_groundwater
+
+end interface
+
+!===============================================================================
+
+! Key ids for clipping
+call field_get_key_id("min_scalar_clipping", kscmin)
+call field_get_key_id("max_scalar_clipping", kscmax)
 
 ! 1. Gas combustion
 !------------------
@@ -91,7 +123,6 @@ if (     ippmod(icod3p).ge.0   &
     .or. ippmod(icoebu).ge.0   &
     .or. ippmod(icolwc).ge.0) then
   call covarp
-  !==========
 endif
 
 ! Number of Diracs for LWC model
@@ -114,14 +145,12 @@ endif
 
 if (ippmod(iccoal).ge.0) then
   call cs_coal_varpos
-  !==================
 endif
 
 ! Pulverized coal combustion coupled with Lagrangian model
 
 if (ippmod(icpl3c).ge.0) then
   call cplvar
-  !==========
 endif
 
 ! 3. Compressible model
@@ -129,17 +158,13 @@ endif
 
 if (ippmod(icompf).ge.0) then
   call cfvarp
-  !==========
 endif
 
 ! 4. Electric arcs model
 !-----------------------
 
-if ( ippmod(ieljou).ge.1 .or.                                     &
-     ippmod(ielarc).ge.1 .or.                                     &
-     ippmod(ielion).ge.1       ) then
-  call elvarp
-  !==========
+if (ippmod(ieljou).ge.1 .or. ippmod(ielarc).ge.1) then
+  call cs_elec_add_variable_fields
 endif
 
 ! 5. Fuel combustion model
@@ -147,7 +172,6 @@ endif
 
 if (ippmod(icfuel).ge.0) then
   call cs_fuel_varpos
-  !==================
 endif
 
 ! 6. Atmospheric model
@@ -155,15 +179,94 @@ endif
 
 if (ippmod(iatmos).ge.0) then
   call atvarp
-  !==========
 endif
 
 ! 7. Cooling towers model
 !------------------------
 
 if (ippmod(iaeros).ge.0) then
-  call ctvarp
-  !==========
+  call ctvarp(ippmod(iatmos))
+endif
+
+! 8. Gas mixtures modelling
+!--------------------------
+
+if (ippmod(igmix).ge.0) then
+
+  if (ippmod(icompf).lt.0) then
+    itherm = 2
+    call add_model_scalar_field('enthalpy', 'Enthalpy', ihm)
+    iscalt = ihm
+  else
+    call field_set_key_int(ivarfl(isca(itempk)), kivisl, 0)
+  endif
+
+  call field_set_key_int(ivarfl(isca(iscalt)), kivisl, 0)
+
+  ! Clipping of mass fractions
+  scminp = 0.d0
+  scmaxp = 1.d0
+
+  if (ippmod(igmix).lt.5) then
+    call add_model_scalar_field('y_o2', 'Y_O2', iscasp(1))
+    f_id = ivarfl(isca(iscasp(1)))
+    call gas_mix_add_species(f_id)
+    call field_set_key_int(f_id, kivisl, 0)
+    call field_set_key_double(f_id, kscmin, scminp)
+    call field_set_key_double(f_id, kscmax, scmaxp)
+
+    call add_model_scalar_field('y_n2', 'Y_N2', iscasp(2))
+    f_id = ivarfl(isca(iscasp(2)))
+    call gas_mix_add_species(f_id)
+    call field_set_key_int(f_id, kivisl, 0)
+    call field_set_key_double(f_id, kscmin, scminp)
+    call field_set_key_double(f_id, kscmax, scmaxp)
+
+    if (ippmod(igmix).eq.3) then
+      call add_model_scalar_field('y_he', 'Y_He', iscasp(3))
+      f_id = ivarfl(isca(iscasp(3)))
+      call gas_mix_add_species(f_id)
+      call field_set_key_int(f_id, kivisl, 0)
+      call field_set_key_double(f_id, kscmin, scminp)
+      call field_set_key_double(f_id, kscmax, scmaxp)
+
+    elseif (ippmod(igmix).eq.4) then
+      call add_model_scalar_field('y_h2', 'Y_H2', iscasp(3))
+      f_id = ivarfl(isca(iscasp(3)))
+      call gas_mix_add_species(f_id)
+      call field_set_key_int(f_id, kivisl, 0)
+      call field_set_key_double(f_id, kscmin, scminp)
+      call field_set_key_double(f_id, kscmax, scmaxp)
+
+    endif
+  else ! ippmod(igmix).eq.5
+
+    call add_model_scalar_field('y_n2', 'Y_N2', iscasp(1))
+    f_id = ivarfl(isca(iscasp(1)))
+    call gas_mix_add_species(f_id)
+    call field_set_key_int(f_id, kivisl, 0)
+    call field_set_key_double(f_id, kscmin, scminp)
+    call field_set_key_double(f_id, kscmax, scmaxp)
+
+    call add_model_scalar_field('y_he', 'Y_He', iscasp(2))
+    f_id = ivarfl(isca(iscasp(2)))
+    call gas_mix_add_species(f_id)
+    call field_set_key_int(f_id, kivisl, 0)
+    call field_set_key_double(f_id, kscmin, scminp)
+    call field_set_key_double(f_id, kscmax, scmaxp)
+
+  endif
+
+  ! MAP to C API
+  call cs_field_pointer_map_gas_mix
+endif
+
+! 9. Groundwater flows
+!---------------------
+
+if (ippmod(idarcy).ge.0) then
+  ! MAP to C API
+  call cs_field_pointer_map_groundwater
 endif
 
 return
